import { useRoutes } from "react-router-dom";
import React, { Suspense } from "react";
import { OBJECT_CATEGORY, FORM_TYPE } from "../constants";
import { t } from "i18next";
import {
  Err404,
  Err500,
  Err501,
  RandomError,
  NetworkError,
  ExternalLoginError,
} from "../views/error/Error";

const Redirect = React.lazy(() => import("../views/login/Redirect"));
const Loader = React.lazy(() => import("../views/loader/loader"));
const Home = React.lazy(() => import("../views/home/Home"));
const Setup = React.lazy(() => import("../views/setup/Setup"));
const PrivateRoute = React.lazy(() =>
  import("../components/layout/private/PrivateRoute")
);
const MainLayout = React.lazy(() =>
  import("../components/layout/main/MainLayout")
);
const CalendarForm = React.lazy(() =>
  import("../views/calendarForm/CalendarForm")
);
const ObjectList = React.lazy(() => import("../views/objectList/ObjectList"));
const ScheduleForm = React.lazy(() =>
  import("../views/scheduleForm/ScheduleForm")
);
const FilterForm = React.lazy(() => import("../views/filterForm/FilterForm"));
const JobnetForm = React.lazy(() => import("../views/jobnetForm/JobnetForm"));
const ObjectVersion = React.lazy(() =>
  import("../views/objectVersion/ObjectVersion")
);
const GeneralSetting = React.lazy(() =>
  import("../views/generalSetting/GeneralSetting")
);
const LockManagement = React.lazy(() =>
  import("../views/lockManagement/LockManagement")
);
const JobExecutionResult = React.lazy(() =>
  import("../views/jobExecutionResult/JobExecutionResult")
);
const JobExecutionManagement = React.lazy(() =>
  import("../views/jobExecutionManagement/JobExecutionManagement")
);

export function MainRoutes() {
  const _Login = (
    <Suspense>
      <PrivateRoute
        element={Loader}
        meta={{
          requiresAuth: false,
          title: t("title-login"),
          externalLogin: true,
        }}
      />
    </Suspense>
  );

  const elements = useRoutes([
    {
      path: "/",
      element: _Login,
    },
    {
      path: "/404",
      element: (
        <PrivateRoute
          element={Err404}
          meta={{
            requiresAuth: false,
            title: t("title-err"),
          }}
        />
      ),
    },
    {
      path: "/error",
      element: (
        <PrivateRoute
          element={RandomError}
          meta={{
            requiresAuth: false,
            title: t("title-err"),
          }}
        />
      ),
    },
    {
      path: "/500",
      element: (
        <PrivateRoute
          element={Err500}
          meta={{
            requiresAuth: false,
            title: t("title-err"),
          }}
        />
      ),
    },
    {
      path: "/501",
      element: (
        <PrivateRoute
          element={Err501}
          meta={{
            requiresAuth: false,
            title: t("title-err"),
          }}
        />
      ),
    },
    {
      path: "/networkErr",
      element: (
        <PrivateRoute
          element={NetworkError}
          meta={{
            requiresAuth: false,
            title: t("title-err"),
          }}
        />
      ),
    },
    {
      path: "/extLoginErr",
      element: (
        <PrivateRoute
          element={ExternalLoginError}
          meta={{
            requiresAuth: false,
            title: t("title-err"),
          }}
        />
      ),
    },
    {
      path: "/redirect",
      element: (
        <Suspense>
          <PrivateRoute
            element={Redirect}
            meta={{
              requiresAuth: false,
              title: t("title-redirect"),
            }}
          />
        </Suspense>
      ),
    },
    {
      path: "/login",
      element: _Login,
    },
    {
      path: "/setup",
      element: (
        <Suspense>
          <PrivateRoute
            element={Setup}
            meta={{
              requiresAuth: false,
              title: t("title-setup"),
            }}
          />
        </Suspense>
      ),
    },
    {
      path: "/",
      element: (
        <Suspense>
          <MainLayout />
        </Suspense>
      ),
      children: [
        {
          path: "home",
          element: (
            <Suspense>
              <PrivateRoute
                element={Home}
                meta={{
                  title: t("title-home"),
                  requiresAuth: true,
                }}
              />
            </Suspense>
          ),
        },
        {
          path: "object-list/calendar/public",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectList}
                meta={{
                  title: t("title-pub-cal-lists"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.CALENDAR}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "object-list/calendar/private",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectList}
                meta={{
                  title: t("title-pri-cal-lists"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.CALENDAR}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "object-list/filter/public",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectList}
                meta={{
                  title: t("title-pub-fil-lists"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.FILTER}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "object-list/filter/private",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectList}
                meta={{
                  title: t("title-pri-fil-lists"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.FILTER}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "object-list/schedule/public",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectList}
                meta={{
                  title: t("title-pub-sch-lists"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.SCHEDULE}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "object-list/schedule/private",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectList}
                meta={{
                  title: t("title-pri-sch-lists"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.SCHEDULE}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "schedule/edit/public/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={ScheduleForm}
                meta={{
                  title: t("title-sch-edi"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.EDIT}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "schedule/edit/private/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={ScheduleForm}
                meta={{
                  title: t("title-sch-edi"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.EDIT}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "object-list/jobnet/public",
          element: (
            <PrivateRoute
              element={ObjectList}
              meta={{
                title: t("title-pub-job-lists"),
                requiresAuth: true,
              }}
              category={OBJECT_CATEGORY.JOBNET}
              publicType={true}
            />
          ),
        },
        {
          path: "object-list/jobnet/private",
          element: (
            <PrivateRoute
              element={ObjectList}
              meta={{
                title: t("title-pri-job-lists"),
                requiresAuth: true,
              }}
              category={OBJECT_CATEGORY.JOBNET}
              publicType={false}
            />
          ),
        },
        {
          path: "calendar/create/public/:id",
          element: (
            <Suspense>
              <PrivateRoute
                element={CalendarForm}
                meta={{
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.CREATE}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "calendar/create/public/",
          element: (
            <Suspense>
              <PrivateRoute
                element={CalendarForm}
                meta={{
                  title: t("title-cal-create"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.CREATE}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "calendar/create/private",
          element: (
            <Suspense>
              <PrivateRoute
                element={CalendarForm}
                meta={{
                  title: t("title-cal-create"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.CREATE}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "calendar/create/private/:id",
          element: (
            <Suspense>
              <PrivateRoute
                element={CalendarForm}
                meta={{
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.CREATE}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "calendar/edit/private/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={CalendarForm}
                meta={{
                  title: t("title-cal-edi"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.EDIT}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "calendar/new-object/private/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={CalendarForm}
                meta={{
                  title: t("title-cal-new-obj"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_OBJECT}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "calendar/new-version/private/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={CalendarForm}
                meta={{
                  title: t("title-cal-new-ver"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_VERSION}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "calendar/edit/public/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={CalendarForm}
                meta={{
                  title: t("title-cal-edi"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.EDIT}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "calendar/new-object/public/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={CalendarForm}
                meta={{
                  title: t("title-cal-new-obj"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_OBJECT}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "calendar/new-version/public/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={CalendarForm}
                meta={{
                  title: t("title-cal-new-ver"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_VERSION}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "filter/create/public",
          element: (
            <Suspense>
              <PrivateRoute
                element={FilterForm}
                meta={{
                  title: t("title-fil-create"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.CREATE}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "filter/edit/private/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={FilterForm}
                meta={{
                  title: t("title-fil-edi"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.EDIT}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "filter/new-object/private/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={FilterForm}
                meta={{
                  title: t("title-fil-new-obj"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_OBJECT}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "filter/new-object/public/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={FilterForm}
                meta={{
                  title: t("title-fil-new-obj"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_OBJECT}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "filter/edit/public/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={FilterForm}
                meta={{
                  title: t("title-fil-edi"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.EDIT}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "filter/create/private",
          element: (
            <Suspense>
              <PrivateRoute
                element={FilterForm}
                meta={{
                  title: t("title-fil-create"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.CREATE}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "schedule/create/public",
          element: (
            <Suspense>
              <PrivateRoute
                element={ScheduleForm}
                meta={{
                  title: t("title-sch-create"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.CREATE}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "schedule/new-object/public/:objectId/:date",
          element: (
            <PrivateRoute
              element={ScheduleForm}
              meta={{
                requiresAuth: true,
                title: t("title-sch-new-obj"),
              }}
              formType={FORM_TYPE.NEW_OBJECT}
              publicType={true}
            />
          ),
        },
        {
          path: "schedule/new-version/public/:objectId/:date",
          element: (
            <PrivateRoute
              element={ScheduleForm}
              meta={{
                title: t("title-sch-new-ver"),
                requiresAuth: true,
              }}
              formType={FORM_TYPE.NEW_VERSION}
              publicType={true}
            />
          ),
        },
        {
          path: "schedule/create/private",
          element: (
            <PrivateRoute
              element={ScheduleForm}
              meta={{
                title: t("title-sch-create"),
                requiresAuth: true,
              }}
              formType={FORM_TYPE.CREATE}
              publicType={false}
            />
          ),
        },
        {
          path: "schedule/new-object/private/:objectId/:date",
          element: (
            <PrivateRoute
              element={ScheduleForm}
              meta={{
                title: t("title-sch-new-obj"),
                requiresAuth: true,
              }}
              formType={FORM_TYPE.NEW_OBJECT}
              publicType={false}
            />
          ),
        },
        {
          path: "schedule/new-version/private/:objectId/:date",
          element: (
            <PrivateRoute
              element={ScheduleForm}
              meta={{
                title: t("title-sch-new-ver"),
                requiresAuth: true,
              }}
              formType={FORM_TYPE.NEW_VERSION}
              publicType={false}
            />
          ),
        },
        {
          path: "jobnet/create/public",
          element: (
            <Suspense>
              <PrivateRoute
                element={JobnetForm}
                meta={{
                  title: t("title-job-create"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.CREATE}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "jobnet/create/private",
          element: (
            <Suspense>
              <PrivateRoute
                element={JobnetForm}
                meta={{
                  title: t("title-job-create"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.CREATE}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "jobnet/edit/public/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={JobnetForm}
                meta={{
                  title: t("title-job-edi"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.EDIT}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "jobnet/edit/private/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={JobnetForm}
                meta={{
                  title: t("title-job-edi"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.EDIT}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "jobnet/new-version/public/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={JobnetForm}
                meta={{
                  title: t("title-job-new-ver"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_VERSION}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "jobnet/new-version/private/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={JobnetForm}
                meta={{
                  title: t("title-job-new-ver"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_VERSION}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "jobnet/new-object/public/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={JobnetForm}
                meta={{
                  title: t("title-job-new-obj"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_OBJECT}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "jobnet/new-object/private/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={JobnetForm}
                meta={{
                  title: t("title-cal-new-obj"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_OBJECT}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "object-version/calendar/public/:objectId",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectVersion}
                meta={{
                  title: t("title-cal-ver"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.CALENDAR}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "object-version/calendar/private/:objectId",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectVersion}
                meta={{
                  title: t("title-cal-ver"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.CALENDAR}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "object-version/filter/public/:objectId",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectVersion}
                meta={{
                  title: t("title-fil-ver"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.FILTER}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "object-version/filter/private/:objectId",
          element: (
            <PrivateRoute
              element={ObjectVersion}
              meta={{
                title: t("title-fil-ver"),
                requiresAuth: true,
              }}
              category={OBJECT_CATEGORY.FILTER}
              publicType={false}
            />
          ),
        },
        {
          path: "filter/new-version/public/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={FilterForm}
                meta={{
                  title: t("title-fil-new-ver"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_VERSION}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "filter/new-version/private/:objectId/:date",
          element: (
            <Suspense>
              <PrivateRoute
                element={FilterForm}
                meta={{
                  title: t("title-fil-new-ver"),
                  requiresAuth: true,
                }}
                formType={FORM_TYPE.NEW_VERSION}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "object-version/schedule/public/:objectId",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectVersion}
                meta={{
                  title: t("title-sch-ver"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.SCHEDULE}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "object-version/schedule/private/:objectId",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectVersion}
                meta={{
                  title: t("title-sch-ver"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.SCHEDULE}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "object-version/jobnet/public/:objectId",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectVersion}
                meta={{
                  title: t("title-job-ver"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.JOBNET}
                publicType={true}
              />
            </Suspense>
          ),
        },
        {
          path: "object-version/jobnet/private/:objectId",
          element: (
            <Suspense>
              <PrivateRoute
                element={ObjectVersion}
                meta={{
                  title: t("title-job-ver"),
                  requiresAuth: true,
                }}
                category={OBJECT_CATEGORY.JOBNET}
                publicType={false}
              />
            </Suspense>
          ),
        },
        {
          path: "general-setting",
          element: (
            <Suspense>
              <PrivateRoute
                element={GeneralSetting}
                meta={{
                  title: t("title-general-setting"),
                  requiresAuth: true,
                }}
              />
            </Suspense>
          ),
        },
        {
          path: "lock-management",
          element: (
            <Suspense>
              <PrivateRoute
                element={LockManagement}
                meta={{
                  title: t("title-lock-management"),
                  requiresAuth: true,
                }}
              />
            </Suspense>
          ),
        },
        {
          path: "job-execution-result",
          element: (
            <Suspense>
              <PrivateRoute
                element={JobExecutionResult}
                meta={{
                  title: t("title-job-exe-result"),
                  requiresAuth: true,
                }}
              />
            </Suspense>
          ),
        },
        {
          path: "job-execution-management",
          element: (
            <Suspense>
              <PrivateRoute
                element={JobExecutionManagement}
                meta={{
                  title: t("title-job-exe-management"),
                  requiresAuth: true,
                }}
              />
            </Suspense>
          ),
        },
        {
          path: "*",
          element: (
            <PrivateRoute
              element={Err404}
              meta={{
                title: t("title-err"),
                requiresAuth: true,
              }}
            />
          ),
        },
      ],
    },
    {
      path: "*",
      element: (
        <PrivateRoute
          element={Err404}
          meta={{
            requiresAuth: false,
            title: "404 Not Found",
          }}
        />
      ),
    },
  ]);

  return elements;
}
