/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
** Copyright (C) 2013 Daiwa Institute of Research Business Innovation Ltd. All Rights Reserved.
** Copyright (C) 2021 Daiwa Institute of Research Ltd. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

#include "common.h"
#include "log.h"
#include "db.h"

#include "jacommon.h"
#include "jalog.h"
#include "jajoblog.h"
#include "jaself.h"
#include "jarunnormal.h"
#include "jarunskip.h"
#include "jarun.h"
#include "jatimeout.h"
#include "sysinfo.h"
#include "jastr.h"

extern unsigned char process_type;
extern int process_num;
char var_file_dir[256] = "/tmp/jobstatus.chk";
const char* var_file_name = "/jobstatus.chk";

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int process_jarun()
{
    int ret;
    DB_RESULT result;
    DB_RESULT result2;
    DB_RESULT result3;
    DB_RESULT result4;
    DB_RESULT session_result;
    DB_RESULT abort_flag_result;

    DB_ROW row;
    DB_ROW row2;
    DB_ROW row3;
    DB_ROW row4;
    DB_ROW session_row;
    DB_ROW abort_flag_row;
    double sec;
    zbx_uint64_t inner_job_id, inner_jobnet_id, inner_jobnet_main_id;
    int method_flag, job_type, test_flag;

    static pid_t pid_check = -1;
    JA_PID result_pid;
    static double next_execution_time_for_session = -1;
    int canSessionExecute = JA_FALSE;
    
    const char *__function_name = "process_jarun";

    ret = FAIL;
    sec = zbx_time();
    result =
        DBselect("select inner_job_id, inner_jobnet_id, method_flag, job_type, test_flag, inner_jobnet_main_id from ja_run_job_table"
                 " where status = %d and method_flag <> %d ", JA_JOB_STATUS_READY, JA_JOB_METHOD_ABORT);

    sec = zbx_time() - sec;
    zabbix_log(LOG_LEVEL_DEBUG,
               "In %s() ja_run_job_table(status = READY): " ZBX_FS_DBL
               " sec.",
               __function_name, sec);

    while (NULL != (row = DBfetch(result)))
    {
        ZBX_STR2UINT64(inner_job_id, row[0]);
        ZBX_STR2UINT64(inner_jobnet_main_id, row[5]);
        abort_flag_result = 
            DBselect("select jobnet_abort_flag from ja_run_jobnet_summary_table"
                " where inner_jobnet_id = "ZBX_FS_UI64, inner_jobnet_main_id);
                
        if(NULL != (abort_flag_row = DBfetch(abort_flag_result))){
            if(atoi(abort_flag_row[0])== JA_JOBENT_ABORT_FLAG_ON){
                DBfree_result(abort_flag_result);
                continue;
            }
        }
        DBfree_result(abort_flag_result);

        ZBX_STR2UINT64(inner_jobnet_id, row[1]);
        method_flag = atoi(row[2]);
        job_type = atoi(row[3]);
        test_flag = atoi(row[4]);
        zabbix_log(LOG_LEVEL_DEBUG,
                   "In %s() inner_job_id: " ZBX_FS_UI64
                   ", inner_jobnet_id: " ZBX_FS_UI64
                   ", method_flag: %d, job_type: %d, test_flag: %d",
                   __function_name, inner_job_id, inner_jobnet_id,
                   method_flag, job_type, test_flag);

        ret = SUCCEED;
        DBbegin();
        result2 = DBselect("select inner_job_id from ja_run_job_table where inner_job_id = " ZBX_FS_UI64 " and status = %d for update", inner_job_id, JA_JOB_STATUS_READY);
        if (NULL == (row2 = DBfetch(result2)))
        {
            DBcommit();
            DBfree_result(result2);
            continue;
        }
        DBfree_result(result2);

        switch (method_flag)
        {
        case JA_JOB_METHOD_NORMAL:
            ja_joblog(JC_JOB_START, inner_jobnet_id, inner_job_id);
            ret = jarun_normal(inner_job_id, job_type, test_flag);
            break;
        case JA_JOB_METHOD_WAIT:
            ret = FAIL;
            break;
        case JA_JOB_METHOD_SKIP:
            ret = jarun_skip(inner_job_id, inner_jobnet_id, job_type);
            break;
        // case JA_JOB_METHOD_ABORT:
        //     ja_log("JARUN300001", inner_jobnet_id, NULL, inner_job_id,
        //            __function_name, inner_job_id);
        //     break;
        case JA_JOB_METHOD_RERUN:
            ja_joblog(JC_JOB_RERUN, inner_jobnet_id, inner_job_id);
            ret = jarun_normal(inner_job_id, job_type, test_flag);
            break;
        default:
            ja_log("JARUN200001", inner_jobnet_id, NULL, inner_job_id,
                   method_flag, inner_job_id, job_type);
            break;
        }

        if (ret == SUCCEED)
        {
            DBcommit();
        }
        else
        {
            DBrollback();
        }
    }
    DBfree_result(result); 
    zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
    return ret;

}

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
void main_jarun_loop()
{
    zabbix_log(LOG_LEVEL_DEBUG,
               "In main_jarun_loop() process_type:'%s' process_num:%d",
               ja_get_process_type_string(process_type), process_num);

    zbx_setproctitle("%s [connecting to the database]",
                     ja_get_process_type_string(process_type));

    const char* lastSlash = strrchr(CONFIG_LOG_FILE, '/');
    
    if (lastSlash != NULL) {
        size_t directoryLength = lastSlash - CONFIG_LOG_FILE;
        zbx_strlcpy(var_file_dir, CONFIG_LOG_FILE, directoryLength+1);
        strcat(var_file_dir, var_file_name);
        var_file_dir[directoryLength+ strlen(var_file_name)] = '\0';
    }
    ja_alarm_watcher("ja_run");
    DBconnect(ZBX_DB_CONNECT_NORMAL);

    for (;;) {
        zbx_setproctitle("%s [processing data]",
                         ja_get_process_type_string(process_type));
        ja_alarm_timeout(CONFIG_RUN_TIMEOUT);
        if (process_jarun() == SUCCEED){
            ja_alarm_timeout(0);
            continue;
        }
        ja_alarm_timeout(0);
        ja_sleep_loop(CONFIG_JARUN_INTERVAL);
    }
}
